/*
# Projeto:
	Demonstração de controle de servo motor usando Displays (LCM) Proculus.

# Descrição:
	Exemplo de controle de um servo motor usando um Display Proculus (UnicView AD).
	A posição do servo é controlada pelo Arduino, e definida através de um destes métodos:

	* Botões e sliders no Display.
	* Potenciômetro
	
	No Display, a posição pode ser atualizada deslizando um slider, ou tocando nos botões
	com posições predefinidas.

# Documentação do Projeto:
 	http://www.victorvision.com.br/projects/arduino-system/library/site/examples/servo/

# Autores:
    * Renan Candido Vieira
	* Andreo de Freitas (revisão)
	* Rafael Moura (revisão)

# Licença:
	Copyright (c) 2020 Proculus Technologies Limited and Victor Vision LTDA

	Permission is hereby granted, free of charge, to any person obtaining a copy
	of this software and associated documentation files (the "Software"), to deal
	in the Software without restriction, including without limitation the rights
	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
	copies of the Software, and to permit persons to whom the Software is
	furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in
	all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
	THE SOFTWARE.
*/

// *******************************************************************************************************************************************
// * Inclusão das bibliotecas necessárias para o projeto
// *******************************************************************************************************************************************
#include <Servo.h>		   // Adiciona biblioteca padrão de controle de servo motores
#include <UnicViewAD.h>  // Adiciona a biblioteca de interface de comunicação serial com Displays Proculus

// *******************************************************************************************************************************************
// * Inicialização do LCM e do Servo
// *******************************************************************************************************************************************
Servo ServoMotor; // Inicializa o servo
LCM Lcm(Serial);  // Inicializa o LCM com o nome "Lcm" e com o parâmetro da Serial de comunicação com o Display

// *******************************************************************************************************************************************
// * Variáveis de configuração (setup)
// *******************************************************************************************************************************************
const long lcmBaudrate = 115200; // Define a taxa de transmissão de dados do Display com o Arduino
const int servoPin = 9;			     // Define o pino de controle do servo

// *******************************************************************************************************************************************
// * Variáveis globais
// *******************************************************************************************************************************************
int positionSetpoint = 0;	   // Variável que recebe o valor da posição desejada do servo
int positionCurrent = 0;	   // Variável que recebe o valor da posição atual do servo
bool updatePosition = false; // Variável que indica se a posição atual do servo pode ser atualizada
bool controlSelect = false;  // Variável que indica se o controle será feito pelo Display (False ou 0) ou pelo potenciômetro (True ou 1)

long int nextPositionWriteTime = 0;		  // Armazena o tempo absoluto para a próxima atualização da posição do servo
const int nextnWriteIntervalInMs = 10; // Espaço de tempo entre os ajustes da posição do servo, em milissegundos

const int positionA0AnalogPin = A1;			// Define qual pino analógico será lido
long int analogReadTime = 0;		        // Armazena o tempo absoluto para a próxima leitura do pino analógico
const int analogReadIntervalInMs = 100; // Espaço de tempo entre as leituras do pino analógico, em milisegundos

// *******************************************************************************************************************************************
// * Inicialização dos objetos LcmVar
// *******************************************************************************************************************************************
LcmVar ToggleControlSelect(100);	  // Declara o VP do botão toggle (habilita/desabilita) que define se o controle é feito pelo potenciômetro ou pelo Display
LcmVar ControlDisplayInput(500);	  // Declara o VP de entrada do valor atribuído à posição desejada do servo
LcmVar PositionNumericDisplay(501); // Declara o VP do mostrador numérico da posição atual do servo

// *******************************************************************************************************************************************
// * Configuração inicial (setup)
// *******************************************************************************************************************************************
void setup()
{
	ServoMotor.attach(servoPin); // Inicia o servo no pino definido
	Lcm.begin();		            // Inicia e configura o baud rate definido no lcm
  Serial.begin(lcmBaudrate);
	updatePosition = true;		  // Inicia a atualização da posição atual do servo como verdadeira
}

// *******************************************************************************************************************************************
// * Programa principal (loop)
// *******************************************************************************************************************************************
void loop()
{
	// *******************************************************************************************************************************************
	// * Seleção do tipo de controle do servo
	// *******************************************************************************************************************************************
	if (ToggleControlSelect.available())             // Verifica se chegou algo do VP "ToggleControlSelect"
	{
		controlSelect = ToggleControlSelect.getData(); // Atribui o valor recebido pelo "ToggleControlSelect" à variável "controlSelect"
	}

	// *******************************************************************************************************************************************
	// * Atribuição da posição do servo
	// *******************************************************************************************************************************************
	if (controlSelect == true)       // Verifica se a variável "controlSelect" é verdadeira
	{
		if (millis() > analogReadTime) // Verifica se "millis()" (tempo atual) é maior que a variável "nextPositionA0WriteTime" (tempo da próxima leitura)
		{
			updatePosition = true;											        // A posição atual do servo deve ser atualizada
			long value = analogRead(positionA0AnalogPin);			  // Lê o potenciômetro no pino analógico
			value = map(value, 0, 1023, 0, 180);							  // A função matemática "map" reescreve o valor lido do potenciômetro (0 a 1023) na escala de 0 a 180
			positionSetpoint = value;										        // Atribui o valor de 0 a 180 à variável "positionSetpoint"
			analogReadTime = millis() + analogReadIntervalInMs; // Atribui o (tempo atual + intervalo de tempo) à variável "nextPositionA0WriteTime" (tempo da próxima leitura)
		}
	}
	else if (ControlDisplayInput.available())           // Verifica se o Display enviou um novo valor de posição desejada
	{
		updatePosition = true;							              // A posição atual do servo deve ser atualizada
		positionSetpoint = ControlDisplayInput.getData(); // Atribui o valor recebido do VP "ControlDisplayInput" à variável "positionSetpoint"
	}

	// *******************************************************************************************************************************************
	// * Ajuste gradual da posição do servo
	// *******************************************************************************************************************************************
	if (updatePosition == true)                      // Verifica se a posição atual do servo deve ser atualizada
	{
		if (millis() > nextPositionWriteTime)          // Verifica se "millis()" (tempo atual) é maior que a variável "nextPositionWriteTime" (tempo da próxima atualização)
		{
			if (positionSetpoint > positionCurrent)      // Verifica se a variável "positionSetpoint" é maior que a variável "positionCurrent"
			{
				positionCurrent++;                         // Incrementa de 1 a variável "positionCurrent"
			}
			else if (positionSetpoint < positionCurrent) // Verifica se a variável "positionSetpoint" é menor que a variável "positionCurrent"
			{
				positionCurrent--;                         // Decrementa de 1 a variável "positionCurrent"
			}

			ServoMotor.write(positionCurrent);							           // Modifica a posição atual do servo para o valor da variável "positionCurrent"
			PositionNumericDisplay.write(positionCurrent);				     // Escreve no LcmVar "CurrentPressureIndicator" o valor da variável "positionCurrent" (envia para o Display)
			nextPositionWriteTime = millis() + nextnWriteIntervalInMs; // Atribui o (tempo atual + intervalo de tempo) à variável "positionWriteIntervalInMs" (tempo da próxima atualização)
		}
	}
	// *******************************************************************************************************************************************
	// * Fim do ajuste da posição do servo
	// *******************************************************************************************************************************************

	if (positionSetpoint == positionCurrent) // Verifica se a variável "positionSetpoint" é igual a variável "positionCurrent"
	{
		updatePosition = false;                // Desliga a atualização da posição atual do servo
	}

	// *******************************************************************************************************************************************
	// * Fim do loop
	// *******************************************************************************************************************************************
}
